﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;
using System.Threading;
using System.Reflection;
using BioAPI;
using BioAPI.Data;


namespace BSPGuti
{
    public class BSPGutiImplemented : IBSP
    {
        private static UUID bspUUID = new UUID(new Guid("db1d7ca9-1984-4ac8-a2fb-3d5c4e43304a"));
        private static string path = @"..\..\..\..\BSPGuti\bin\Debug\BSPGuti.dll";
        private BSPSchema schema = new BSPSchema(bspUUID, null, path, null, "BSPGuti 1.0", "GUTI", null, null, null, null, 0, 0, 0, 0,
            0, 0, 0, 0, 0, 0, null, null, null, null);

        private List<BFPSchema> bfplist = new List<BFPSchema>();
        private List<IBFP> bfpinstanced = new List<IBFP>();
        private List<UnitSchema> unitlist = new List<UnitSchema>();
        private ISensor mySensorUnit = null;
        private IComparison myComparisonUnit = null;
        private IProcessing myProcessingUnit = null;
        private IArchive myArchiveUnit = null;
        private BIR biometricReference = null;
        private byte[] acBioInstance = null;
        private bool isBSPLoaded = false;
        private Dictionary<int, UnitSchema> dictionary = new Dictionary<int, UnitSchema>();
        private ComparisonGuti compGuti = new ComparisonGuti();
        private ProcessingGuti procGuti = new ProcessingGuti();

        public BSPSchema Schema
        {
            get
            {
                return schema;
            }
        }

        public BIR BiometricReference
        {
            get
            {
                return biometricReference;
            }
        }

        public byte[] ACBioInstance
        {
            get
            {
                return acBioInstance;
            }
        }

        public void BSPAttach(string bioAPIVersion, ACBioParameters acBioParameters, List<UnitListElement> unitListElements,
            List<SecurityProfileType> securityProfileList)
        {
            if (isBSPLoaded == false)
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrBspNotLoaded,
                     Properties.Resources.stringBSPNotLoaded);

            for (int n = 0; n < unitListElements.Count; n++) //Iteration inside the list of units giving by the application
            {
                UnitListElement element = unitListElements[n];
                UnitSchema unitSchema = unitlist.Where(u => (u.UnitID == element.UnitID)).First();

                if (unitSchema.UnitCategory == UnitCategoryType.Sensor) //For each category
                {
                    #region Sensor
                    if (unitSchema.UnitManagerUUID == unitSchema.BSPUUID) //BSP managing
                    {
                        Assembly a = Assembly.GetExecutingAssembly();
                        foreach (Type type in a.GetTypes())
                        {
                            if (type.IsClass && typeof(ISensor).IsAssignableFrom(type) && type.IsNested)
                            {
                                mySensorUnit = (ISensor)Activator.CreateInstance(type);
                                if (mySensorUnit.UnitSchema.VendorInformation == dictionary[unitSchema.UnitID].VendorInformation)
                                {
                                    break;
                                }
                                else { }
                            }
                        }
                    }
                    else //BFP managing
                    {
                        for (int b = 0; b < bfplist.Count; b++)
                        {
                            if (unitSchema.UnitManagerUUID.ID == bfplist[b].BFPUUID.ID)
                            {
                                Assembly a = Assembly.LoadFrom(bfplist[b].Path);
                                foreach (Type type in a.GetTypes())
                                {
                                    if (type.IsClass && typeof(ISensor).IsAssignableFrom(type) && type.IsNested)
                                    {
                                        mySensorUnit = (ISensor)Activator.CreateInstance(type);
                                        if (mySensorUnit.UnitSchema.VendorInformation == dictionary[unitSchema.UnitID].VendorInformation)
                                        {
                                            break;
                                        }
                                        else { }
                                    }
                                }
                            }
                        }
                    }
                    #endregion
                }

                else if (unitSchema.UnitCategory == UnitCategoryType.Archive)
                {
                    #region Archive
                    if (unitSchema.UnitManagerUUID == unitSchema.BSPUUID) 
                    {
                        Assembly a = Assembly.GetExecutingAssembly();
                        foreach (Type type in a.GetTypes())
                        {
                            if (type.IsClass && typeof(IArchive).IsAssignableFrom(type) && type.IsNested)
                            {
                                myArchiveUnit = (IArchive)Activator.CreateInstance(type);
                                if (myArchiveUnit.UnitSchema.VendorInformation == dictionary[unitSchema.UnitID].VendorInformation)
                                {
                                    break;
                                }
                                else { }
                            }
                        }
                    }
                    else 
                    {
                        for (int b = 0; b < bfplist.Count; b++)
                        {
                            if (unitSchema.UnitManagerUUID.ID == bfplist[b].BFPUUID.ID)
                            {
                                Assembly a = Assembly.LoadFrom(bfplist[b].Path);
                                foreach (Type type in a.GetTypes())
                                {
                                    if (type.IsClass && typeof(IArchive).IsAssignableFrom(type) && type.IsNested)
                                    {
                                        myArchiveUnit = (IArchive)Activator.CreateInstance(type);
                                        if (myArchiveUnit.UnitSchema.VendorInformation == dictionary[unitSchema.UnitID].VendorInformation)
                                        {
                                            break;
                                        }
                                        else { }
                                    }
                                }
                            }
                        }
                    }
                    #endregion
                }

                else if (unitSchema.UnitCategory == UnitCategoryType.Processing)
                {
                    #region Processing
                    if (unitSchema.UnitManagerUUID == unitSchema.BSPUUID) 
                    {
                        Assembly a = Assembly.GetExecutingAssembly();
                        foreach (Type type in a.GetTypes())
                        {
                            if (type.IsClass && typeof(IProcessing).IsAssignableFrom(type) && type.IsNested)
                            {
                                myProcessingUnit = (IProcessing)Activator.CreateInstance(type);
                                if (myProcessingUnit.UnitSchema.VendorInformation == dictionary[unitSchema.UnitID].VendorInformation)
                                {
                                    break;
                                }
                                else { }
                            }
                        }
                    }
                    else 
                    {
                        for (int b = 0; b < bfplist.Count; b++)
                        {
                            if (unitSchema.UnitManagerUUID.ID == bfplist[b].BFPUUID.ID)
                            {
                                Assembly a = Assembly.LoadFrom(bfplist[b].Path);
                                foreach (Type type in a.GetTypes())
                                {
                                    if (type.IsClass && typeof(IProcessing).IsAssignableFrom(type) && type.IsNested)
                                    {
                                        myProcessingUnit = (IProcessing)Activator.CreateInstance(type);
                                        if (myProcessingUnit.UnitSchema.VendorInformation == dictionary[unitSchema.UnitID].VendorInformation)
                                        {
                                            break;
                                        }
                                        else { }
                                    }
                                }
                            }
                        }
                    }
                    #endregion
                }

                else
                {
                    #region Comparison
                    if (unitSchema.UnitManagerUUID == unitSchema.BSPUUID) 
                    {
                        Assembly a = Assembly.GetExecutingAssembly();
                        foreach (Type type in a.GetTypes())
                        {
                            if (type.IsClass && typeof(IComparison).IsAssignableFrom(type) && type.IsNested)
                            {
                                myComparisonUnit = (IComparison)Activator.CreateInstance(type);
                                if (myComparisonUnit.UnitSchema.VendorInformation == dictionary[unitSchema.UnitID].VendorInformation)
                                {
                                    break;
                                }
                                else { }
                            }
                        }
                    }
                    else 
                    {
                        for (int b = 0; b < bfplist.Count; b++)
                        {
                            if (unitSchema.UnitManagerUUID.ID == bfplist[b].BFPUUID.ID)
                            {
                                Assembly a = Assembly.LoadFrom(bfplist[b].Path);
                                foreach (Type type in a.GetTypes())
                                {
                                    if (type.IsClass && typeof(IComparison).IsAssignableFrom(type) && type.IsNested)
                                    {
                                        myComparisonUnit = (IComparison)Activator.CreateInstance(type);
                                        if (myComparisonUnit.UnitSchema.VendorInformation == dictionary[unitSchema.UnitID].VendorInformation)
                                        {
                                            break;
                                        }
                                        else { }
                                    }
                                }
                            }
                        }
                    }
                    #endregion
                }
            }
        }

        public void BSPDetach()
        {
            if (isBSPLoaded == false)
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrBspNotLoaded,
                     Properties.Resources.stringBSPNotLoaded);

        }

        public void BSPLoad(BFPEventCallback bioAPINotifyCallback, BFPEnumerationCallback bfpEnumeration)
        {
            isBSPLoaded = true;
            bfplist = bfpEnumeration();

            for (int i = 0; i < bfplist.Count; i++)
            {
                Assembly a = Assembly.LoadFrom(bfplist[i].Path);
                foreach (Type type in a.GetTypes())
                {
                    if (type.IsClass && typeof(IBFP).IsAssignableFrom(type))
                    {
                        bfpinstanced.Add((IBFP)Activator.CreateInstance(type));
                    }
                }
            }

            if (bfpinstanced.Count > 0)
            {
                for (int i = 0; i < bfpinstanced.Count; i++)
                {
                    bfpinstanced[i].BFPLoad(bioAPINotifyCallback);
                    List<UnitSchema> auxlist = bfpinstanced[i].QueryUnits();
                    for (int n = 0; n < auxlist.Count; n++)
                    {
                        unitlist.Add(auxlist[n]);
                    }
                }
            }

            unitlist.Add(compGuti.UnitSchema);
            unitlist.Add(procGuti.UnitSchema);

            for (int i = 0; i < unitlist.Count; i++)
            {
                unitlist[i].UnitID = i;
                dictionary.Add(i, unitlist[i]);
                unitlist[i].BSPUUID = bspUUID;
            }
        }

        public byte[] ControlUnit(int unitID, int controlCode, byte[] inputData)
        {
            if (isBSPLoaded == false)
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrBspNotLoaded,
                     Properties.Resources.stringBSPNotLoaded);

            return null;
        }

        #region Unload, CheckQuality, Enrols
        public void BSPUnload()
        {

        }

        public byte CheckQuality(BIR inputBIR, DataFormat qualityAlgorithmID)
        {
            if (isBSPLoaded == false)
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrBspNotLoaded,
                     Properties.Resources.stringBSPNotLoaded);

            return 0;
        }

        public UUID Enrol(BIR capturedBIR, BIR referenceTemplate, List<Purpose> purpose, BiometricSubtype subtype, DataFormat outputFormat,
            byte[] payload, int timeout, List<ResultOptions> options)
        {
            if (isBSPLoaded == false)
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrBspNotLoaded,
                     Properties.Resources.stringBSPNotLoaded);

            return null;
        }

        public UUID Enrol(List<BIR> capturedBIRs, BIR referenceTemplate, List<Purpose> purpose, BiometricSubtype subtype,
            DataFormat outputFormat, byte[] payload, int timeout, List<ResultOptions> options)
        {
            if (isBSPLoaded == false)
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrBspNotLoaded,
                     Properties.Resources.stringBSPNotLoaded);

            return null;
        }

        public UUID Enrol(int numberOfSamples, BIR referenceTemplate, List<Purpose> purpose, BiometricSubtype subtype, DataFormat outputFormat,
            byte[] payload, int timeout, List<ResultOptions> options)
        {
            if (isBSPLoaded == false)
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrBspNotLoaded,
                     Properties.Resources.stringBSPNotLoaded);

            return null;
        }

        public UUID Enrol(BIR capturedBIR, UUID referenceID, List<Purpose> purpose, BiometricSubtype subtype, DataFormat outputFormat,
            byte[] payload, int timeout, List<ResultOptions> options)
        {
            if (isBSPLoaded == false)
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrBspNotLoaded,
                     Properties.Resources.stringBSPNotLoaded);

            return null;
        }

        public UUID Enrol(List<BIR> capturedBIRs, UUID referenceID, List<Purpose> purpose, BiometricSubtype subtype, DataFormat outputFormat,
            byte[] payload, int timeout, List<ResultOptions> options)
        {
            if (isBSPLoaded == false)
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrBspNotLoaded,
                     Properties.Resources.stringBSPNotLoaded);

            return null;
        }

        public UUID Enrol(int numberOfSamples, UUID referenceID, List<Purpose> listPurpose, BiometricSubtype subtype, DataFormat outputFormat,
            byte[] payload, int timeout, List<ResultOptions> options)
        {
            if (isBSPLoaded == false)
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrBspNotLoaded,
                     Properties.Resources.stringBSPNotLoaded);

            return null;
        }

        #endregion

        #region IdentifyAggregated, QueryBFPs
        public List<Candidate> IdentifyAggregated(int maxFMRrequested, BiometricSubtype subtype, bool binning, int maxResults, int timeout,
            List<ResultOptions> options)
        {
            if (isBSPLoaded == false)
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrBspNotLoaded,
                     Properties.Resources.stringBSPNotLoaded);

            return null;
        }
        public List<BFPListElement> QueryBFPs()
        {
            if (isBSPLoaded == false)
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrBspNotLoaded,
                     Properties.Resources.stringBSPNotLoaded);

            return null;
        }
        public List<BFPListElement> QueryBFPs(List<UnitCategoryType> unitCategories)
        {
            if (isBSPLoaded == false)
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrBspNotLoaded,
                     Properties.Resources.stringBSPNotLoaded);

            return null;
        }
        #endregion

        public List<UnitSchema> QueryUnits()
        {
            if (isBSPLoaded == false)
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrBspNotLoaded,
                     Properties.Resources.stringBSPNotLoaded);

            return unitlist;
        }

        public List<UnitSchema> QueryUnits(List<UnitCategoryType> unitCategories)
        {
            if (isBSPLoaded == false)
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrBspNotLoaded,
                     Properties.Resources.stringBSPNotLoaded);

            List<UnitSchema> unitlistcategories = new List<UnitSchema>();
            foreach (UnitSchema unit in unitlist)
            {
                if (unit.UnitCategory == unitCategories[0])
                {
                    unitlistcategories.Add(unit);
                }
            }
            return unitlistcategories;
        }

        #region VerifyAggregateds
        public bool VerifyAggregated(int MaxFMRrequested, BIR ReferenceTemplate, BiometricSubtype Subtype, int Timeout,
            List<ResultOptions> Options)
        {
            return false;
        }
        public bool VerifyAggregated(int MaxFMRrequested, BIR ProcessedBIR, UUID ReferenceKey, BiometricSubtype Subtype, int Timeout,
            List<ResultOptions> Options)
        {
            return false;
        }
        public bool VerifyAggregated(int MaxFMRrequested, UUID ReferenceKey, BiometricSubtype Subtype, int Timeout,
            List<ResultOptions> Options)
        {
            return false;
        }
        #endregion

        public void SubscribeToGUIEvents(GUISelectEventCallback guiSelectEventCallback, GUIStateEventCallback
            guiStateEventCallback, GUIProgressEventCallback guiProgressEventCallback)
        {
            if (isBSPLoaded == false)
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrBspNotLoaded,
                     Properties.Resources.stringBSPNotLoaded);

        }

        public void UnsubscribeFromGUIEvents()
        {
            if (isBSPLoaded == false)
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrBspNotLoaded,
                     Properties.Resources.stringBSPNotLoaded);

        }

        ///////////////////////////////////////////////////////////////////////////////////////////

        #region From here, extensions of Sensor, Archive, Comparison and Processing

        public UnitSchema UnitSchema
        {
            get
            {
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrFunctionNotSupported,
                    Properties.Resources.stringFunctionNotSupported);
            }
            set
            {
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrFunctionNotSupported,
                    Properties.Resources.stringFunctionNotSupported);
            }
        }

        public byte[] AuxiliaryData
        {
            get
            {
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrFunctionNotSupported,
                    Properties.Resources.stringFunctionNotSupported);
            }
        }

        public UnitIndicatorStatus IndicatorStatus
        {
            get
            {
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrFunctionNotSupported,
                    Properties.Resources.stringFunctionNotSupported);
            }
        }

        public BIR Capture(List<Purpose> Purpose, BiometricSubtype Subtype, DataFormat OutputFormat, int Timeout, List<ResultOptions> Options)
        {
            return mySensorUnit.Capture(Purpose, Subtype, OutputFormat, Timeout, Options);
        }

        public void Calibrate(int timeout)
        {
            mySensorUnit.Calibrate(timeout);
        }

        public void SetIndicatorStatus(UnitIndicatorStatus IndicatorStatus)
        {
            mySensorUnit.SetIndicatorStatus(IndicatorStatus);
        }

        public void SetPowerMode(UnitPowerMode PowerMode)
        {
            mySensorUnit.SetPowerMode(PowerMode);
        }

        ////////

        public BIR CreateTemplate(BIR CapturedBIR, BIR ReferenceTemplate, DataFormat OutputFormat, byte[] Payload)
        {
            return myProcessingUnit.CreateTemplate(CapturedBIR, ReferenceTemplate, OutputFormat, Payload);
        }

        public BIR CreateTemplate(List<BIR> CapturedBIRs, BIR ReferenceTemplate, DataFormat OutputFormat, byte[] payload)
        {
            return myProcessingUnit.CreateTemplate(CapturedBIRs, ReferenceTemplate, OutputFormat, Payload);
        }

        public BIR Process(BIR CapturedBIR, DataFormat OutputFormat)
        {
            return myProcessingUnit.Process(CapturedBIR, OutputFormat);
        }

        public BIR Process(BIR CapturedBIR, List<BIR> AuxiliaryBIRs, DataFormat OutputFormat)
        {
            return myProcessingUnit.Process(CapturedBIR, AuxiliaryBIRs, OutputFormat);
        }

        ////////

        public int FMRAchieved
        {
            get
            {
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrFunctionNotSupported,
                    Properties.Resources.stringFunctionNotSupported);
            }
        }

        public BIR AdaptedBIR
        {
            get
            {
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrFunctionNotSupported,
                    Properties.Resources.stringFunctionNotSupported);
            }
        }

        public byte[] Payload
        {
            get
            {
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIErrFunctionNotSupported,
                    Properties.Resources.stringFunctionNotSupported);
            }
        }

        public List<Candidate> Identify(int MaxFMRrequested, BIR ProcessedBIR, bool Binning, int MaxResults, int Timeout)
        {
            return myComparisonUnit.Identify(MaxFMRrequested, ProcessedBIR, Binning, MaxResults, Timeout);
        }

        public List<Candidate> Identify(int MaxFMRrequested, BIR ProcessedBIR, List<BIR> AuxiliaryBIRs, bool Binning, int MaxResults, int TimeOut)
        {
            return myComparisonUnit.Identify(MaxFMRrequested, ProcessedBIR, AuxiliaryBIRs, Binning, MaxResults, TimeOut);
        }

        public void PresetIdentifyPopulation(IdentifyPopulation Population)
        {
            myComparisonUnit.PresetIdentifyPopulation(Population);
        }

        public bool Verify(int MaxFMRrequested, BIR ProcessedBIR, BIR ReferenceTemplate, List<ResultOptions> Options)
        {
            return myComparisonUnit.Verify(MaxFMRrequested, ProcessedBIR, ReferenceTemplate, Options);
        }

        public bool Verify(int MaxFMRrequested, BIR ProcessedBIR, BIR ReferenceTemplate, List<BIR> AuxiliaryBIRs, List<ResultOptions> Options)
        {
            return myComparisonUnit.Verify(MaxFMRrequested, ProcessedBIR, ReferenceTemplate, AuxiliaryBIRs, Options);
        }

        ////////

        public void CloseDatabase()
        {
            myArchiveUnit.CloseDatabase();
        }

        public void DeleteBIR(UUID Key)
        {
            myArchiveUnit.DeleteBIR(Key);
        }

        public BIR GetSingleBIR(UUID Key)
        {
            return myArchiveUnit.GetSingleBIR(Key);
        }

        public List<UUID> ListUUIDs()
        {
            return myArchiveUnit.ListUUIDs();
        }

        public IdentifyPopulation NewIdentifyPopulation()
        {
            return myArchiveUnit.NewIdentifyPopulation();
        }

        public IdentifyPopulation NewIdentifyPopulation(List<UUID> UuidList)
        {
            return myArchiveUnit.NewIdentifyPopulation(UuidList);
        }

        public IdentifyPopulation NewIdentifyPopulation(byte[] Query)
        {
            return myArchiveUnit.NewIdentifyPopulation(Query);
        }

        public void OpenDatabase(byte[] DatabaseID, BIRDatabaseAccess Access)
        {
            myArchiveUnit.OpenDatabase(DatabaseID, Access);
        }

        public UUID StoreBIR(BIR BiometricReference)
        {
            return myArchiveUnit.StoreBIR(BiometricReference);
        }

        public void StoreBIR(BIR BiometricReference, UUID Key)
        {
            myArchiveUnit.StoreBIR(BiometricReference, Key);
        }

        public UUID StoreBIR(BIR BiometricReference, byte[] AuxiliaryData)
        {
            return myArchiveUnit.StoreBIR(BiometricReference, AuxiliaryData);
        }

        public void StoreBIR(BIR BiometricReference, byte[] AuxiliaryData, UUID Key)
        {
            myArchiveUnit.StoreBIR(BiometricReference, AuxiliaryData, Key);
        }
        #endregion

        public class ProcessingGuti : IProcessing
        {
            UnitSchema unitSchema = new UnitSchema(bspUUID, bspUUID, 0, UnitCategoryType.Processing, null, "ProcessingGuti", null, null,
                null, null, null, null, null, false, 0, 0, null);
            byte[] acBioInstance = null;

            public UnitSchema UnitSchema
            {
                get
                {
                    return unitSchema;
                }
                set
                {
                    unitSchema = value;
                }
            }
            public byte[] ACBioInstance
            {
                get
                {
                    return acBioInstance;
                }
            }

            public BIR CreateTemplate(BIR capturedBIR, BIR referenceTemplate, DataFormat outputFormat, byte[] payload)
            {
                return null;
            }

            public BIR CreateTemplate(List<BIR> capturedBIRs, BIR referenceTemplate, DataFormat outputFormat, byte[] payload)
            {
                return null;
            }

            public BIR Process(BIR capturedBIR, DataFormat outputFormat)
            {
                //*****//
                //Here it would come the call to Process method from the selected algorithm, which gives back a byteArray with the
                //biometric information processed

                byte[] birProcessed = null;

                BIR bir = new BIR(capturedBIR.PatronFormat, capturedBIR.PatronHeaderVersion, capturedBIR.CBEFFVersion,
                        capturedBIR.BDBFormat, capturedBIR.BDBEncription, capturedBIR.BDBIntegrity, capturedBIR.BDBBiometricType,
                        capturedBIR.BDBBiometricSubtype, capturedBIR.BDBChallengeResponse, capturedBIR.BDBCreationDate, capturedBIR.BDBIndex,
                        ProcessedLevel.Processed, capturedBIR.BDBProduct, capturedBIR.BDBPurpose, capturedBIR.BDBQuality,
                        capturedBIR.BDBValidityPeriod, capturedBIR.BIRCreationDate, capturedBIR.BIRCreator, capturedBIR.BIRIndex,
                        capturedBIR.BIRPayload, capturedBIR.BIRValidityPeriod, capturedBIR.SBFormat, birProcessed, capturedBIR.SBData);

                return bir;
            }

            private byte[] GetByteArray(int integer)
            {
                byte[] intBytes = BitConverter.GetBytes(integer);
                if (BitConverter.IsLittleEndian)
                    Array.Reverse(intBytes);
                byte[] result = intBytes;
                return result;
            }

            public BIR Process(BIR capturedBIR, List<BIR> auxiliaryBIRs, DataFormat outputFormat)
            {
                return null;
            }

            public static Image Convert(Bitmap oldBitmap)
            {
                using (var ms = new System.IO.MemoryStream())
                {
                    oldBitmap.Save(ms, System.Drawing.Imaging.ImageFormat.Gif);
                    ms.Position = 0;
                    return Image.FromStream(ms);
                }
            }

            public static byte[] Combine(byte[] first, byte[] second, byte[] third, byte[] fourth)
            {
                byte[] ret = new byte[first.Length + second.Length + third.Length + fourth.Length];
                Buffer.BlockCopy(first, 0, ret, 0, first.Length);
                Buffer.BlockCopy(second, 0, ret, first.Length, second.Length);
                Buffer.BlockCopy(third, 0, ret, first.Length + second.Length, third.Length);
                Buffer.BlockCopy(fourth, 0, ret, first.Length + second.Length + third.Length, fourth.Length);
                return ret;
            }

        }

        public class ComparisonGuti : IComparison
        {
            UnitSchema unitSchema = new UnitSchema(bspUUID, bspUUID, 0, UnitCategoryType.Comparison, null, "ComparisonGuti", null, null,
                null, null, null, null, null, false, 0, 0, null);
            byte[] acBioInstance = null;
            int fmrAchieved = 0;
            BIR adaptedBIR = null;
            byte[] payload = null;

            public UnitSchema UnitSchema
            {
                get
                {
                    return unitSchema;
                }
                set
                {
                    unitSchema = value;
                }
            }
            public byte[] ACBioInstance
            {
                get
                {
                    return acBioInstance;
                }
            }
            public int FMRAchieved
            {
                get
                {
                    return fmrAchieved;
                }
            }
            public BIR AdaptedBIR
            {
                get
                {
                    return adaptedBIR;
                }
            }
            public byte[] Payload
            {
                get
                {
                    return payload;
                }
            }

            public List<Candidate> Identify(int maxFMRrequested, BIR processedBIR, bool binning, int maxResults, int yimeout)
            {
                return null;
            }

            public List<Candidate> Identify(int maxFMRrequested, BIR processedBIR, List<BIR> auxiliaryBIRs, bool binning,
                int maxResults, int timeout)
            {
                return null;
            }

            public void PresetIdentifyPopulation(IdentifyPopulation population)
            {

            }

            public bool Verify(int maxFMRrequested, BIR processedBIR, BIR referenceTemplate, List<ResultOptions> options)
            {
                //*****//
                //Here it would come the call to Verify method from the selected algorithm, which gives back an int with the
                //result of the matching process.

                int result = 100;

                if (result > maxFMRrequested) { return true; }
                else { return false; }
            }

            public bool Verify(int maxFMRrequested, BIR processedBIR, BIR referenceTemplate, List<BIR> auxiliaryBIRs,
                List<ResultOptions> options)
            {
                return false;
            }
        }
    }
}
