﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Management;
using System.IO;
using System.Reflection;
using BioAPI;
using BioAPI.Data;

namespace Beta1
{
    public partial class FormPrincipal : Form
    {
        private string path3 = @"..\..\..\..\Framework\bin\Debug\Framework.dll"; 
        private const string path2 = @"..\..\..\..\BSPGuti\bin\Debug\BSPGuti.dll";
        private const string path4 = @"..\..\..\..\BFPSensor\bin\Debug\BFPSensor.dll";
        private const string path5 = @"..\..\..\..\BFPArchive\bin\Debug\BFPArchive.dll";

        private BSPSchema schema = new BSPSchema(new UUID(new Guid("db1d7ca9-1984-4ac8-a2fb-3d5c4e43304a")), null, path2, null, 
            "BSPGuti 1.0", "GUTI", null, null, null, null, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, null, null, null, null);
        private BFPSchema bfpSchemaSensor = new BFPSchema(new UUID(new Guid("db1d7ca9-1984-4ac8-a2fb-3d5c4e43304b")), 
            UnitCategoryType.Sensor, null, path4, null, "BFPSensor 1.0", "GUTI", null, null, null, null, null);
        private BFPSchema bfpSchemaArchive = new BFPSchema(new UUID(new Guid("db1d7ca9-1984-4ac8-a2fb-3d5c4e43304c")), 
            UnitCategoryType.Archive, null, path5, null, "BFPArchive 1.0", "GUTI", null, null, null, null, null);

        private IFramework fwtest = null;
        private List<BSPSchema> listBSP = new List<BSPSchema>();
        private List<BFPSchema> listBFP = new List<BFPSchema>();
        private List<UnitSchema> listUnit = new List<UnitSchema>();
        private List<UnitListElement> listUnitElement = new List<UnitListElement>();
        private AttachedSession attachsession = null;

        public FormPrincipal()
        {
            InitializeComponent();
        }

        private DialogResult ShowImage(byte[] byteImage)
        {
            DialogResult result = DialogResult.No;

            if (checkBox1.Checked)
            {
                MemoryStream ms = new MemoryStream(byteImage);
                FormSetImage SI = new FormSetImage(ms);
                result = SI.ShowDialog();
            }
            else
            {
                FormSetImage SI = new FormSetImage(null);
                result = SI.ShowDialog();
            }

            return result;
        }

        private static string GetLast(string source, int tailLength)
        {
            if (tailLength >= source.Length)
                return source;
            return source.Substring(source.Length - tailLength);
        }

        //Callback from the BSP
        public bool ShowMessage(UUID bspUUID, int unitID, GUIOperation operation, GUISuboperation suboperation,
            Purpose purpose, GUIMoment moment, byte suboperationProgress, List<GUIBitmap> bitmaps, string text, GUIResponse response)
        {
            CheckForIllegalCrossThreadCalls = false;
            label1.Text = text;
            return true;
        }

        //Callback to the BSP
        public List<BFPSchema> BFPEnumeration()
        {
            listBFP = fwtest.EnumBFPs();
            return listBFP;
        }

        private void buttonInstall_Click(object sender, EventArgs e)
        {
            //Dynamical load for the Framework, from the dll of FrameworkImplemented. 
            Assembly a = Assembly.LoadFrom(path3);
            foreach (Type type in a.GetTypes())
            {
                if (type.IsClass && typeof(IFramework).IsAssignableFrom(type))
                {
                    fwtest = (IFramework)Activator.CreateInstance(type);
                    break;
                }
            }
            Thread t1 = new Thread(FirstThreadFunction);
            t1.Start();
        }

        private void FirstThreadFunction()
        {
            CheckForIllegalCrossThreadCalls = false;
            buttonInstall.Enabled = false;
            panel3.BackgroundImage = Properties.Resources.ProgressWheel;
            label1.Text = Properties.Resources.stringWork;

            //This is just a simulation for the installer
            fwtest.InstallBSP(schema, false);
            fwtest.InstallBFP(bfpSchemaSensor, false);
            fwtest.InstallBFP(bfpSchemaArchive, false);
            fwtest.Init("4.0");
            listBSP = fwtest.EnumBSPs();

            //The comboBox is filled with all the BSPs that are installed in the system, through a call to EnumBSPs method from Framework 
            for (int i = 0; i < listBSP.Count; i++)
            {
                comboBox1.Items.Add(listBSP[i].Vendor + " " + listBSP[i].ProductVersion);
            }

            panel3.BackgroundImage = null;
            buttonBSP.Enabled = true;
            panel1.Enabled = true;
            label1.Text = Properties.Resources.stringBSP;
        }

        private void buttonBSP_Click(object sender, EventArgs e)
        {
            buttonBSP.Enabled = false;
            panel1.Enabled = false;

            if (comboBox1.SelectedIndex == -1)
            {
                MessageBox.Show(Properties.Resources.stringNoBSP, "", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                buttonBSP.Enabled = true;
                panel1.Enabled = true;
            }
            else
            {
                //Call to BSPLoad method from Framework (not from BSP, because application cannot talk with it directly)
                fwtest.BSPLoad(listBSP[comboBox1.SelectedIndex].BSPUUID, null, BFPEnumeration, null);

                Thread t2 = new Thread(SecondThreadFunction);
                t2.Start();
            }
        }

        public void SecondThreadFunction()
        {
            panel3.BackgroundImage = Properties.Resources.ProgressWheel;
            label1.Text = Properties.Resources.stringWork;

            //The ComboBox is filled with all the units that are installed in the system, through a call to EnumBSPs method from Framework.
            //The units that are located in BFPs are found by the BSP through the callback to the BSP, BFPEnumeration.
            listUnit = fwtest.QueryUnits(listBSP[comboBox1.SelectedIndex].BSPUUID);

            for (int i = 0; i < listUnit.Count; i++)
            {
                if(listUnit[i].UnitCategory == UnitCategoryType.Sensor) 
                comboBox2.Items.Add(listUnit[i].VendorInformation + " " + listUnit[i].FirmwareVersion);
            }

            panel3.BackgroundImage = null;
            panel2.Enabled = true;
            label1.Text = "";
            buttonAttach.Enabled = true;
            label1.Text = Properties.Resources.stringUnit;
        }

        private void buttonAttach_Click(object sender, EventArgs e)
        {
            if (comboBox2.SelectedIndex == -1)
            {
                MessageBox.Show(Properties.Resources.stringNoUnit, "", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                panel2.Enabled = true;
            }
            else
            {
                Thread t3 = new Thread(ThirdThreadFunction);
                t3.Start();
            }
        }

        public void ThirdThreadFunction()
        {
            panel3.BackgroundImage = Properties.Resources.ProgressWheel;
            label1.Text = Properties.Resources.stringWork;
            CheckForIllegalCrossThreadCalls = false;
            int sensorIndex = 0, archiveIndex = 0, processingIndex = 0, comparisonIndex = 0;
            string sensorName = null;

            //Each unit has its unique index
            for (int i = 0; i < listUnit.Count; i++)
            {
                if (listUnit[i].UnitCategory == UnitCategoryType.Archive)
                {
                    archiveIndex = listUnit[i].UnitID;
                }
                else if (listUnit[i].UnitCategory == UnitCategoryType.Comparison)
                {
                    comparisonIndex = listUnit[i].UnitID;
                }
                else if (listUnit[i].UnitCategory == UnitCategoryType.Processing)
                {
                    processingIndex = listUnit[i].UnitID;
                }
                else if ((listUnit[i].VendorInformation + " " + listUnit[i].FirmwareVersion).ToString() == comboBox2.SelectedItem.ToString())
                {
                    sensorIndex = listUnit[i].UnitID;
                    sensorName = listUnit[i].VendorInformation;
                }
            }

            listUnitElement.Add(new UnitListElement(UnitCategoryType.Sensor, sensorIndex));
            listUnitElement.Add(new UnitListElement(UnitCategoryType.Archive, archiveIndex));
            listUnitElement.Add(new UnitListElement(UnitCategoryType.Processing, processingIndex));
            listUnitElement.Add(new UnitListElement(UnitCategoryType.Comparison, comparisonIndex));

            //Then, with the selected units, the application makes a call to BSPAttach method from Framework (not from BSP), and 
            //gets back an object from AttachedSession type. With this object, the application could make calls to BSP methods, 
            //without talk directly with it.
            attachsession = fwtest.BSPAttach(listBSP[comboBox1.SelectedIndex].BSPUUID, null, null, listUnitElement, null);

            List<UUID> listUUID = attachsession.ListUUIDs();
            for (int i = 0; i < listUUID.Count; i++)
            {
                comboBox3.Items.Add(GetLast(listUUID[i].ID.ToString(), 4));
            }

            panel3.BackgroundImage = null;
            label1.Text = "";
            buttonAttach.Enabled = false;
            buttonCapture.Enabled = true;
            buttonCompare.Enabled = true;
            checkBox1.Enabled = true;
            panel2.Enabled = false;
            buttonSelectionID.Enabled = true;
        }

        private void buttonCapture_Click(object sender, EventArgs e)
        {
            panel4.Enabled = false;
            buttonSelectionID.Enabled = false;
            comboBox3.Items.Clear();
            buttonCapture.Enabled = false;
            buttonCompare.Enabled = false;
            checkBox1.Enabled = false;

            if (comboBox2.SelectedIndex == -1)
            {
                MessageBox.Show(Properties.Resources.stringNoUnit, "", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                buttonCapture.Enabled = true;
                buttonCompare.Enabled = true;
                checkBox1.Enabled = true;
            }
            else
            {
                Thread t4 = new Thread(FourthThreadFunction);
                t4.Start();
            }
        }

        public void FourthThreadFunction()
        {
            if (checkBox1.Checked)
            {
                //Subscription to callbacks
                attachsession.SubscribeToGUIEvents(null, null, ShowMessage); 
            }
            else
            {
                //Unsubscription to callbacks
                attachsession.UnsubscribeFromGUIEvents(); 
            }

            label1.Text = Properties.Resources.stringFinger;

            //Call to Capture (monolithic system) instead to Enrol
            BIR bir = attachsession.Capture(null, BiometricSubtype.RightIndexFinger, null, 40000, null);
            //byte[] birByteArray = bir.BIRToByteArray();

            panel3.BackgroundImage = Properties.Resources.ProgressWheel;
            label1.Text = Properties.Resources.stringWork;

            if (bir != null)
            {
                byte[] image = bir.BDBData;
                panel3.BackgroundImage = null;
                label1.Text = " ";

                DialogResult result = ShowImage(image);
                if (result == DialogResult.Yes) 
                {
                    //Process
                    bir = attachsession.Process(bir, null);

                    //Store
                    UUID reference = attachsession.StoreBIR(bir);
                    MessageBox.Show(Properties.Resources.stringStored + GetLast(reference.ID.ToString(), 4), Properties.Resources.stringSuccess, 
                        MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else { }
            }
            else
            {
                MessageBox.Show(Properties.Resources.stringFailure, Properties.Resources.stringFail, MessageBoxButtons.OK, 
                    MessageBoxIcon.Error);
            }

            panel3.BackgroundImage = null;
            label1.Text = " ";
            comboBox2.SelectedIndex = -1;
            buttonAttach.Enabled = true;
            panel2.Enabled = true;
            checkBox1.Checked = false;
            checkBox1.Enabled = false;
            buttonSelectionID.Enabled = false;
        }

        private void buttonCompare_Click(object sender, EventArgs e)
        {
            buttonCapture.Enabled = false;
            buttonCompare.Enabled = false;
            buttonSelectionID.Enabled = false;
            checkBox1.Enabled = false;

            if (comboBox2.SelectedIndex == -1)
            {
                MessageBox.Show(Properties.Resources.stringNoUnit, "", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                buttonCompare.Enabled = true;
                buttonCapture.Enabled = true;
                checkBox1.Enabled = true;
            }
            else if (comboBox3.SelectedIndex == -1)
            {
                MessageBox.Show(Properties.Resources.stringNoID, "", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                label1.Text = Properties.Resources.stringID;
                buttonCompare.Enabled = true;
                buttonCapture.Enabled = true;
                checkBox1.Enabled = true;
            }
            else
            {
                if (comboBox3.Items.Count != 0)
                {
                    Thread t5 = new Thread(FifthThreadFunction);
                    t5.Start();
                }
                else
                {
                    panel3.BackgroundImage = null;
                    MessageBox.Show(Properties.Resources.stringNoUsers, "", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    buttonAttach.Enabled = true;
                    panel2.Enabled = true;
                    comboBox2.SelectedIndex = -1;
                    checkBox1.Checked = false;
                    checkBox1.Enabled = false;
                }
            }
        }

        public void FifthThreadFunction()
        {
            CheckForIllegalCrossThreadCalls = false;
            panel4.Enabled = false;

            UUID referenceKey = new UUID(new Guid("00000000-0000-0000-0000-00000000" + comboBox3.SelectedItem.ToString()));

            label1.Text = Properties.Resources.stringFinger;
            BIR bir = attachsession.Capture(null, BiometricSubtype.RightIndexFinger, null, 40000, null);

            if (bir != null)
            {
                //Process
                BIR BIRToCompare = attachsession.Process(bir, null);
                label1.Text = " ";

                panel3.BackgroundImage = Properties.Resources.ProgressWheel;
                label1.Text = Properties.Resources.stringWork;

                //Verify
                bool result = attachsession.Verify(80, BIRToCompare, attachsession.GetSingleBIR(referenceKey), null);
                label1.Text = result ? Properties.Resources.stringCorrect : Properties.Resources.stringFalse;
                panel3.BackgroundImage = result ? Properties.Resources.GreenTick : Properties.Resources.RedCross;
            }
            else
            {
                panel3.BackgroundImage = null;
                MessageBox.Show(Properties.Resources.stringFailure, Properties.Resources.stringFail, MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }

            buttonAttach.Enabled = true;
            panel2.Enabled = true;
            comboBox2.SelectedIndex = -1;
            comboBox3.Items.Clear();
            checkBox1.Checked = false;
            checkBox1.Enabled = false;
            buttonSelectionID.Enabled = false;
        }

        private void buttonSelectionID_Click(object sender, EventArgs e)
        {
            panel4.Enabled = true;
        }
    }
}
