﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace BioAPI.Data
{
    public class ACBioParameters
    {
        /// <summary>
        /// <param name="Challenge"/>
        /// Challenge from the validator of a biometric verification when ACBio is used. This value shall be sent to the field controlValue 
        /// of type ACBioContentInformation in ACBio instances.
        /// </param>
        /// <param name="InitialBPUIOIndexOutput"/>
        /// The initial value of BPU IO index which is to be assigned to the output from the BioAPI Unit , BFP, or BSP when the ACBio instances 
        /// are generated. The range between InitialBPUIOIndexOutput and SupremumBPUIOIndexOutput shall be divided into the number of BSP Units 
        /// and BFPs which are attached to the BSP, and assigned to the BSP Units and BSPs.
        /// </param>
        /// <param name="SupremumBPUIOIndexOutput"/>
        /// The supremum of BPU IO indexes which are to be assigned to the output from the BioAPI Unit , BFP, or BSP when the ACBio instances 
        /// are generated.
        /// </param>
        /// </summary>

        public int[] Challenge { get; private set; }
        public int[] InitialBPUIOIndexOutput { get; private set; }
        public int[] SupremumBPUIOIndexOutput { get; private set; }

        public ACBioParameters(int[] challenge, int[] initialBPUIOIndexOutput, int[] supremumBPUIOIndexOutput)
        {
            Challenge = challenge;
            InitialBPUIOIndexOutput = initialBPUIOIndexOutput;
            SupremumBPUIOIndexOutput = supremumBPUIOIndexOutput;
        }
    }
}
