﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace BioAPI.Data
{

    /// <summary>
    /// Represents the record in the component registry that defines properties of the BFP installed in the 
    /// system. For security reasons, no methods to modify the existing BFPProperties object are provided.
    /// Instead, an application should use the corresponding method of the DataFactory to create new 
    /// BFPProperties containing modified data.
    /// </summary>
    /// <author>
    /// See CREDITS.txt
    /// </author>
    [Serializable()]
    public class BFPSchema
    {
        public UUID BFPUUID { get; private set; }
        public UnitCategoryType BFPCategory { get; private set; }
        public string BFPDescription { get; private set; }
        public string Path { get; private set; }
        public string SpecVersion { get; private set; }
        public string ProductVersion { get; private set; }
        public string Vendor { get; private set; }
        public sbyte[] BFPProperty { get; private set; }
        public List<DataFormat> BFPSupportedFormats { get; private set; }
        public List<BiometricType> FactorsMask { get; private set; }
        public UUID FwPropertyID { get; private set; }
        public byte[] FwProperty { get; private set; }

        public BFPSchema
            (UUID bfpUUID, UnitCategoryType bfpCategory, string bfpDescription, string path, string specVersion, string productVersion,
             string vendor, sbyte[] bfpProperty, List<DataFormat> bfpSupportedFormats, List<BiometricType> factorsMask,
             UUID fwPropertyID, byte[] fwProperty)
        {
            BFPUUID = bfpUUID;
            BFPCategory = bfpCategory;
            BFPDescription = bfpDescription;
            Path = path;
            SpecVersion = specVersion;
            ProductVersion = productVersion;
            Vendor = vendor;
            BFPProperty = bfpProperty;
            BFPSupportedFormats = bfpSupportedFormats;
            FactorsMask = factorsMask;
            FwPropertyID = fwPropertyID;
            FwProperty = fwProperty;
        }
    }
}
