﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace BioAPI.Data
{

    /// <summary>
    /// Represents BIR. No methods to modify the existing BIR object are provided. An application should use
    /// the corresponding method of the DataFactory to create new BIR containing modified data.
    /// Depending on the way the BIR object was created, the BIR is either bound to the BSP or is unbound.
    /// Output of biometric operation normally creates the BIR bound to the BSP. Bound BIR is the BIR 
    /// encapsulated in the BSP with the lifetime limited by the lifetime of the AttachSession that created it.
    /// The bound BIR can be extracted from a BSP:
    /// -implicitly, by calling either of properties BiometricData or SecurityBlock.
    /// -explicitly, by calling the Unbind() method.
    /// </summary>
    /// <author>
    /// See CREDITS.txt
    /// </author>
    public class BIR
    {
        public DataFormat PatronFormat { get; set; }
        public byte CBEFFVersion { get; set; }
        public byte PatronHeaderVersion { get; set; }
        public DataFormat BDBFormat { get; set; }
        public bool BDBEncription { get; set; }
        public bool BDBIntegrity { get; set; }
        public BiometricType BDBBiometricType { get; set; }
        public BiometricSubtype BDBBiometricSubtype { get; set; }
        public byte[] BDBChallengeResponse { get; set; }
        public Date BDBCreationDate { get; set; }
        public byte[] BDBIndex { get; set; }
        public ProcessedLevel BDBProcessedLevel { get; set; }
        public DataFormat BDBProduct { get; set; }
        public Purpose BDBPurpose { get; set; }
        public byte BDBQuality { get; set; }
        public List<Date> BDBValidityPeriod { get; set; }
        public Date BIRCreationDate { get; set; }
        public byte[] BIRCreator { get; set; }
        public byte[] BIRIndex { get; set; }
        public byte[] BIRPayload { get; set; }
        public List<Date> BIRValidityPeriod { get; set; }
        public DataFormat SBFormat { get; set; }
        public byte[] BDBData { get; set; }
        public byte[] SBData { get; set; }

        public BIR
            (DataFormat patronFormat, byte cbeffVersion, byte patronHeaderVersion, DataFormat bdbFormat, bool bdbEncription, bool bdbIntegrity, 
             BiometricType bdbBiometricType, BiometricSubtype bdbBiometricSubtype, byte[] bdbChallengeResponse, Date bdbCreationDate, 
             byte[] bdbIndex, ProcessedLevel bdbProcessedLevel, DataFormat bdbProduct, Purpose bdbPurpose, byte bdbQuality, 
             List<Date> bdbValidityPeriod, Date birCreationDate, byte[] birCreator, byte[] birIndex, byte[] birPayload, 
             List<Date> birValidityPeriod, DataFormat sbFormat, byte[] bdbData, byte[] sbData)
        {
            PatronFormat = patronFormat;
            CBEFFVersion = cbeffVersion;
            PatronHeaderVersion = patronHeaderVersion;
            BDBFormat = bdbFormat;
            BDBEncription = bdbEncription;
            BDBIntegrity = bdbIntegrity;
            BDBBiometricType = bdbBiometricType;
            BDBBiometricSubtype = bdbBiometricSubtype;
            BDBChallengeResponse = bdbChallengeResponse;
            BDBCreationDate = bdbCreationDate;
            BDBIndex = bdbIndex;
            BDBProcessedLevel = bdbProcessedLevel;
            BDBProduct = bdbProduct;
            BDBPurpose = bdbPurpose;
            BDBQuality = bdbQuality;
            BDBValidityPeriod = bdbValidityPeriod;
            BIRCreationDate = birCreationDate;
            BIRCreator = birCreator;
            BIRIndex = birIndex;
            BIRPayload = birPayload;
            BIRValidityPeriod = birValidityPeriod;
            SBFormat = sbFormat;
            BDBData = bdbData;
            SBData = sbData;
        }

        /// <summary>
        /// Fills in the BIR data from a byte array coded as a ISO/IEC 19785 record
        /// </summary>
        /// <param name="record">
        /// the byte array containing the CBEFF record
        /// </param>
        public void BIRFromByteArray(byte[] record)
        {
 
        }

        /// <summary>
        /// Serializes a BIR record so as to provide it as a byte array representing the CBEFF information
        /// </summary>
        /// <returns>
        /// The byte array containing the CBEFF information
        /// </returns>
        /*public byte[] BIRToByteArray()
        {
           List<byte[]> listArrays = new List<byte[]>();

            #region Byte[] creation
            //PatronFormat
            #region PatronFormat
            byte[] patronFormatOwner = new byte[2];
            byte[] patronFormatType = new byte[2];
            if (PatronFormat != null)
            {
                patronFormatOwner = BitConverter.GetBytes(PatronFormat.Owner);
                patronFormatType = BitConverter.GetBytes(PatronFormat.Type);
            }
            else
            {
                patronFormatOwner[0] = 0;
                patronFormatOwner[1] = 0;
                patronFormatType[0] = 0;
                patronFormatType[1] = 0;
            }
            #endregion

            //CBEFFVersion
            byte[] cbeffVersion = new byte[1];
            cbeffVersion[0] = CBEFFVersion;

            //PatronHeaderVersion
            byte[] patronHeaderVersion = new byte[1];
            patronHeaderVersion[0] = PatronHeaderVersion;

            //FieldPresence
            byte[] fieldPresence = new byte[4];

            //BDBFormat
            #region BDBFormat

            byte[] bdbFormatOwner = new byte[2];
            byte[] bdbFormatType = new byte[2];

            if (BDBFormat != null)
            {
                bdbFormatOwner = BitConverter.GetBytes(BDBFormat.Owner);
                bdbFormatType = BitConverter.GetBytes(BDBFormat.Type);
            }
            else
            {
                bdbFormatOwner[0] = 0;
                bdbFormatOwner[1] = 0;
                bdbFormatType[0] = 0;
                bdbFormatType[1] = 0;
            }
            #endregion

            //BDBEncription
            byte[] bdbEncription = new byte[1];
            if (BDBEncription) { bdbEncription[0] = 1; }
            else { bdbEncription[0] = 0; }

            //BDBIntegrity 
            byte[] bdbIntegrity = new byte[1];
            if (BDBIntegrity) { bdbIntegrity[0] = 1; }
            else { bdbIntegrity[0] = 0; }

            //BDBBiometricType
            #region BiometricType 

            byte[] bdbBiometricType = new byte[3];

            switch(BDBBiometricType)
            {
                case (BiometricType.NoValueAvailable):
                    bdbBiometricType[0] = 0;
                    bdbBiometricType[1] = 0;
                    bdbBiometricType[2] = 0;
                    break;
                case (BiometricType.MultipleBiometricTypes):
                    bdbBiometricType[0] = 0;
                    bdbBiometricType[1] = 0;
                    bdbBiometricType[2] = 1;
                    break;
                case (BiometricType.Face):
                    bdbBiometricType[0] = 0;
                    bdbBiometricType[1] = 0;
                    bdbBiometricType[2] = 2;
                    break;
                case (BiometricType.Voice):
                    bdbBiometricType[0] = 0;
                    bdbBiometricType[1] = 0;
                    bdbBiometricType[2] = 4;
                    break;
                case (BiometricType.Finger):
                    bdbBiometricType[0] = 0;
                    bdbBiometricType[1] = 0;
                    bdbBiometricType[2] = 8;
                    break;
                case (BiometricType.Iris):
                    bdbBiometricType[0] = 0;
                    bdbBiometricType[1] = 0;
                    bdbBiometricType[2] = 16;
                    break;
                case (BiometricType.Retina):
                    bdbBiometricType[0] = 0;
                    bdbBiometricType[1] = 0;
                    bdbBiometricType[2] = 32;
                    break;
                case (BiometricType.HandGeometry):
                    bdbBiometricType[0] = 0;
                    bdbBiometricType[1] = 0;
                    bdbBiometricType[2] = 64;
                    break;
                case (BiometricType.SignatureOrSign):
                    bdbBiometricType[0] = 0;
                    bdbBiometricType[1] = 0;
                    bdbBiometricType[2] = 128;
                    break;
                case (BiometricType.Keystroke):
                    bdbBiometricType[0] = 0;
                    bdbBiometricType[1] = 1;
                    bdbBiometricType[2] = 0;
                    break;
                case (BiometricType.LipMovement):
                    bdbBiometricType[0] = 0;
                    bdbBiometricType[1] = 2;
                    bdbBiometricType[2] = 0;
                    break;
                case (BiometricType.Gait):
                    bdbBiometricType[0] = 0;
                    bdbBiometricType[1] = 16;
                    bdbBiometricType[2] = 0;
                    break;
                case (BiometricType.Vein):
                    bdbBiometricType[0] = 0;
                    bdbBiometricType[1] = 32;
                    bdbBiometricType[2] = 0;
                    break;
                case (BiometricType.DNA):
                    bdbBiometricType[0] = 0;
                    bdbBiometricType[1] = 64;
                    bdbBiometricType[2] = 0;
                    break;
                case (BiometricType.Ear):
                    bdbBiometricType[0] = 0;
                    bdbBiometricType[1] = 128;
                    bdbBiometricType[2] = 0;
                    break;
                case (BiometricType.Foot):
                    bdbBiometricType[0] = 1;
                    bdbBiometricType[1] = 0;
                    bdbBiometricType[2] = 0;
                    break;
                case (BiometricType.Scent):
                    bdbBiometricType[0] = 2;
                    bdbBiometricType[1] = 0;
                    bdbBiometricType[2] = 0;
                    break;
            }
            #endregion

            //BDBBiometricSubtype 
            #region BiometricSubtype

            byte[] bdbBiometricSubtype = new byte[1];

            switch (BDBBiometricSubtype)
            {
                case BiometricSubtype.NoValueAvailable:
                    bdbBiometricSubtype[0] = 0;
                    break;
                case BiometricSubtype.Left:
                    bdbBiometricSubtype[0] = 1;
                    break;
                case BiometricSubtype.Right:
                    bdbBiometricSubtype[0] = 2;
                    break;
                case BiometricSubtype.LeftThumb:
                    bdbBiometricSubtype[0] = 5;
                    break;
                case BiometricSubtype.LeftIndexFinger:
                    bdbBiometricSubtype[0] = 9;
                    break;
                case BiometricSubtype.LeftMiddleFinger:
                    bdbBiometricSubtype[0] = 17;
                    break;
                case BiometricSubtype.LeftRingFinger:
                    bdbBiometricSubtype[0] = 33;
                    break;
                case BiometricSubtype.LeftLittleFinger:
                    bdbBiometricSubtype[0] = 65;
                    break;
                case BiometricSubtype.RightThumb:
                    bdbBiometricSubtype[0] = 6;
                    break;
                case BiometricSubtype.RightIndexFinger:
                    bdbBiometricSubtype[0] = 10;
                    break;
                case BiometricSubtype.RightMiddleFinger:
                    bdbBiometricSubtype[0] = 18;
                    break;
                case BiometricSubtype.RightRingFinger:
                    bdbBiometricSubtype[0] = 34;
                    break;
                case BiometricSubtype.RightLittleFinger:
                    bdbBiometricSubtype[0] = 66;
                    break;
                case BiometricSubtype.LeftPalm:
                    bdbBiometricSubtype[0] = 133;
                    break;
                case BiometricSubtype.LeftBackOfHand:
                    bdbBiometricSubtype[0] = 137;
                    break;
                case BiometricSubtype.LeftWrist:
                    bdbBiometricSubtype[0] = 145;
                    break;
                case BiometricSubtype.RightPalm:
                    bdbBiometricSubtype[0] = 134;
                    break;
                case BiometricSubtype.RightBackOfHand:
                    bdbBiometricSubtype[0] = 138;
                    break;
                case BiometricSubtype.RightWrist:
                    bdbBiometricSubtype[0] = 146;
                    break;
            }
            #endregion

            //BDBChallengeResponse 
            #region BDBChallengeResponse

            byte[] challengeResponseLength = new byte[2];

            if (BDBChallengeResponse != null)
            {
                challengeResponseLength = BitConverter.GetBytes(BDBChallengeResponse.Length);
            }
            else
            {
                challengeResponseLength[0] = 0;
                challengeResponseLength[1] = 0;
            }

            byte[] challengeResponse = BDBChallengeResponse;
            #endregion

            //BDBCreationDate 
            #region BDBCreationDate

            string date;
            byte[] bdbCreationDateLength = new byte[1];
            byte[] bdbCreationDate;

            if(BDBCreationDate != null)
            {
                date = BDBCreationDate.DayOfMonth.ToString() + "-" + BDBCreationDate.Month.ToString() + "-" + BDBCreationDate.Year.ToString();
                bdbCreationDateLength = BitConverter.GetBytes(date.Length);
                bdbCreationDate = GetBytes(date);
            }
            else
            {
                bdbCreationDateLength[0] = 0;
                bdbCreationDate = null;
            }

            #endregion

            //BDBIndex 
            #region BDBIndex

            byte[] bdbIndexLength = new byte[1];

            if(BDBIndex != null)
            {
                bdbIndexLength = BitConverter.GetBytes(BDBIndex.Length);
            }
            else
            {
                bdbIndexLength[0] = 0;
            }

            byte[] bdbIndex = BDBIndex;
            #endregion

            //BDBProcessedLevel
            #region BDBProcessedLevel
            byte[] bdbProcessedLevel = new byte[1];
            switch (BDBProcessedLevel)
            {
                case (ProcessedLevel.Raw):
                    bdbProcessedLevel[0] = 1;
                    break;
                case (ProcessedLevel.Intermediate):
                    bdbProcessedLevel[0] = 2;
                    break;
                case (ProcessedLevel.Processed):
                    bdbProcessedLevel[0] = 3;
                    break;
            }
            #endregion

            //BDBProduct 
            #region BDBFormat

            byte[] productFormatOwner = new byte[2];
            byte[] productFormatType = new byte[2];

            if (BDBProduct != null)
            {
                productFormatOwner = BitConverter.GetBytes(BDBProduct.Owner);
                productFormatType = BitConverter.GetBytes(BDBProduct.Type);
            }
            else
            {
                productFormatOwner[0] = 0;
                productFormatOwner[1] = 0;
                productFormatType[0] = 0;
                productFormatType[1] = 0;
            }
            #endregion

            //BDBPurpose 
            #region BDBPurpose 

            byte[] bdbPurpose = new byte[1];

            switch (BDBPurpose)
            {
                case(Purpose.NoPurposeAvailable):
                    bdbPurpose[0] = 0;
                    break;
                case (Purpose.Verify):
                    bdbPurpose[0] = 1;
                    break;
                case (Purpose.Identify):
                    bdbPurpose[0] = 2;
                    break;
                case (Purpose.Enrol):
                    bdbPurpose[0] = 3;
                    break;
                case (Purpose.EnrolForVerificationOnly):
                    bdbPurpose[0] = 4;
                    break;
                case (Purpose.EnrolForIdentificacionOnly):
                    bdbPurpose[0] = 5;
                    break;
                case (Purpose.Audit):
                    bdbPurpose[0] = 6;
                    break;
            }
            #endregion

            //BDBQuality 
            byte[] bdbQuality = new byte[1];
            bdbQuality[0] = BDBQuality;

            //BDBValidityPeriod 
            #region BDBValidityPeriod

            string validityPeriod = null;
            byte[] bdbValidityPeriod;
            byte[] bdbValidityPeriodLength = new byte[1];

            if (BDBValidityPeriod != null)
            {
               bdbValidityPeriodLength = BitConverter.GetBytes(validityPeriod.Length);

               if (BDBValidityPeriod.Count > 1)
               {
                   validityPeriod = BDBValidityPeriod[0].DayOfMonth.ToString() + "-" + BDBValidityPeriod[0].Month.ToString() + "-"
                       + BDBValidityPeriod[0].Year.ToString() + "---" + BDBValidityPeriod[1].DayOfMonth.ToString() + "-"
                       + BDBValidityPeriod[1].Month.ToString() + "-" + BDBValidityPeriod[1].Year.ToString();
               }
               else
               {
                   validityPeriod = BDBValidityPeriod[0].DayOfMonth.ToString() + "-" + BDBValidityPeriod[0].Month.ToString() + "-"
                      + BDBValidityPeriod[0].Year.ToString() + "---" + BDBValidityPeriod[0].Hour.ToString() + "-"
                      + BDBValidityPeriod[0].Minute.ToString() + "-" + BDBValidityPeriod[0].Second.ToString();
               }

               bdbValidityPeriod = GetBytes(validityPeriod);
            }

            else
            {
                bdbValidityPeriodLength[0] = 0;
                bdbValidityPeriod = null;
            }

            #endregion

            //BIRCreationDate 
            #region CreationDate

            string creationDate = null; ;
            byte[] birCreationDateLength = new byte[1];
            byte[] birCreationDate;

            if(BIRCreationDate != null)
            {
                creationDate = BIRCreationDate.DayOfMonth.ToString() + "-" + BIRCreationDate.Month.ToString() + "-"
                   + BIRCreationDate.Year.ToString() + "---" + BIRCreationDate.Hour.ToString() + "-"
                   + BIRCreationDate.Minute.ToString() + "-" + BIRCreationDate.Second.ToString();

                birCreationDateLength = BitConverter.GetBytes(creationDate.Length);
                birCreationDate = GetBytes(creationDate);
            }
            else
            {
                birCreationDateLength[0] = 0;
                birCreationDate = null;
            }

            #endregion

            //BIRCreator 
            #region BIRCreator
            byte[] birCreatorLength = new byte[1];

            if(BIRCreator != null)
            {
               birCreatorLength = BitConverter.GetBytes(BIRCreator.Length);
            }
            else
            {
                birCreatorLength[0] = 0;
            }

            byte[] birCreator = BIRCreator;
            #endregion

            //BIRIndex 
            #region BIRIndex

            byte[] birIndexLength = new byte[1];

            if(BIRIndex != null)
            {
                birIndexLength = BitConverter.GetBytes(BIRIndex.Length);
            }
            else
            {
                birIndexLength[0] = 0;
            }

            byte[] birIndex = BIRIndex;
            #endregion

            //BIRPayload
            #region BIRPayload
 
            byte[] birPayloadLength = new byte[1];

            if(BIRPayload != null)
            {
                birPayloadLength = BitConverter.GetBytes(BIRPayload.Length);
            }
            else
            {
                birPayloadLength[0] = 0;
            }

            byte[] birPayload = BIRPayload;
            #endregion

            //BIRValidityPeriod
            #region BIRValidityPeriod

            string validityPeriodBIR = null;
            byte[] birValidityPeriod;
            byte[] birValidityPeriodLength = new byte[1];

            if (BIRValidityPeriod != null)
            {
                if (BIRValidityPeriod.Count > 1)
                {
                    validityPeriod = BIRValidityPeriod[0].DayOfMonth.ToString() + "-" + BIRValidityPeriod[0].Month.ToString() + "-"
                        + BIRValidityPeriod[0].Year.ToString() + "---" + BIRValidityPeriod[1].DayOfMonth.ToString() + "-"
                        + BIRValidityPeriod[1].Month.ToString() + "-" + BIRValidityPeriod[1].Year.ToString();
                }
                else
                {
                    validityPeriod = BIRValidityPeriod[0].DayOfMonth.ToString() + "-" + BIRValidityPeriod[0].Month.ToString() + "-"
                       + BIRValidityPeriod[0].Year.ToString() + "---" + BIRValidityPeriod[0].Hour.ToString() + "-"
                       + BIRValidityPeriod[0].Minute.ToString() + "-" + BIRValidityPeriod[0].Second.ToString();
                }

                birValidityPeriodLength = BitConverter.GetBytes(validityPeriodBIR.Length);
                birValidityPeriod = GetBytes(validityPeriodBIR);
            }
            else
            {
                birValidityPeriodLength[0] = 0;
                birValidityPeriod = null;
            }

            #endregion

            //SBFormat
            #region SBFormat

            byte[] sbFormatOwner = new byte[2];
            byte[] sbFormatType = new byte[2];

            if (SBFormat != null)
            {
                sbFormatOwner = BitConverter.GetBytes(SBFormat.Owner);
                sbFormatType = BitConverter.GetBytes(SBFormat.Type);
            }
            else
            {
                sbFormatOwner[0] = 0;
                sbFormatOwner[1] = 0;
                sbFormatType[0] = 0;
                sbFormatType[1] = 0;
            }
            #endregion

            //NumChildren
            byte[] numChildren = new byte[1];
            numChildren[0] = 0;

            //BDBData 
            #region BDBData

            byte[] bdbDataLength = new byte[1];

            if(BDBData != null)
            {
                bdbDataLength = BitConverter.GetBytes(BDBData.Length);
            }
            else
            {
                bdbDataLength[0] = 0;
            }

            byte[] bdbData = BDBData;
            #endregion

            //SBData
            #region SBData

            byte[] sbDataLength = new byte[1];

            if (SBData != null)
            {
                sbDataLength = BitConverter.GetBytes(SBData.Length);
            }
            else
            {
                sbDataLength[0] = 0;
            }

            byte[] sbData = SBData;
            #endregion

            //BIRLength
            byte[] length = new byte[4];

            //Two bytes for each short, and the sum of all the lengths of the byte arrays.
            int sbFormatLength = 0, sbDataExists = 0;
            if (SBFormat != null)
            {
                sbFormatLength = 4;
            }
            if (SBData != null)
            {
                sbDataExists = 4;
            }
            int totalLength = (2 + 2 + 4 + patronHeaderVersion.Length + cbeffVersion.Length + 3 + bdbFormatOwner.Length + bdbFormatType.Length
                + bdbEncription.Length + bdbIntegrity.Length + bdbBiometricType.Length + bdbBiometricSubtype.Length + 2 + 
                BitConverter.ToInt16(challengeResponseLength, 0) + 1 + BitConverter.ToInt(bdbCreationDateLength, 0) + 2 + 
                BitConverter.ToInt32(bdbIndexLength, 0) + bdbProcessedLevel.Length + 2 + 2 + bdbPurpose.Length + bdbQuality.Length
                + 1 + BitConverter.ToInt32(bdbValidityPeriodLength, 0) + 1 + BitConverter.ToInt32(birCreationDateLength, 0) + 2 + 
                BitConverter.ToInt32(birCreatorLength, 0) + 2 + BitConverter.ToInt32(birIndexLength, 0) + 2 +
                BitConverter.ToInt32(birPayloadLength, 0) + 1 + BitConverter.ToInt32(birValidityPeriodLength, 0) + sbFormatLength 
                + 1 + 4 + BitConverter.ToInt32(bdbDataLength, 0) + sbDataExists + BitConverter.ToInt32(sbDataLength, 0));

            #endregion

            #region List adding
            listArrays.Add(patronFormatOwner);
            listArrays.Add(patronFormatType);
            listArrays.Add(length);
            listArrays.Add(cbeffVersion);
            listArrays.Add(patronHeaderVersion);
            listArrays.Add(fieldPresence);
            listArrays.Add(bdbFormatOwner);
            listArrays.Add(bdbFormatType);
            listArrays.Add(bdbEncription);
            listArrays.Add(bdbBiometricType);
            listArrays.Add(bdbBiometricSubtype);
            listArrays.Add(challengeResponseLength);
            listArrays.Add(challengeResponse);
            listArrays.Add(bdbCreationDateLength);
            listArrays.Add(bdbCreationDate);
            listArrays.Add(bdbIndexLength);
            listArrays.Add(bdbIndex);
            listArrays.Add(bdbProcessedLevel);
            listArrays.Add(productFormatOwner);
            listArrays.Add(productFormatType);
            listArrays.Add(bdbPurpose);
            listArrays.Add(bdbQuality);
            listArrays.Add(bdbValidityPeriodLength);
            listArrays.Add(bdbValidityPeriod);
            listArrays.Add(birCreationDateLength);
            listArrays.Add(birCreationDate);
            listArrays.Add(birCreatorLength);
            listArrays.Add(birCreator);
            listArrays.Add(birIndexLength);
            listArrays.Add(birIndex);
            listArrays.Add(birPayloadLength);
            listArrays.Add(birPayload);
            listArrays.Add(birValidityPeriodLength);
            listArrays.Add(birValidityPeriod);
            listArrays.Add(sbFormatOwner);
            listArrays.Add(sbFormatType);
            listArrays.Add(numChildren);
            listArrays.Add(bdbDataLength);
            listArrays.Add(bdbData);
            listArrays.Add(sbDataLength);
            listArrays.Add(sbData);            
            #endregion

            byte[] birToByteArray = new byte[totalLength];

            int dstOffset = 0;
            for (int i = 0; i < listArrays.Count; i++)
            {
                dstOffset = dstOffset + listArrays[i].Length;
                Buffer.BlockCopy(listArrays[i], 0, birToByteArray, dstOffset, listArrays[i].Length);
            }

            return birToByteArray;
        }*/

        public byte[] BIRToByteArray()
        {
            return null;
        }

        /// <summary>
        /// Removes all the information in the current BIR, leaving it empty for a next use
        /// </summary>
        public void Destroy()
        {
 
        }

        private static byte[] GetBytes(string str)
        {
            byte[] bytes = new byte[str.Length * sizeof(char)];
            System.Buffer.BlockCopy(str.ToCharArray(), 0, bytes, 0, bytes.Length);
            return bytes;
        }
    }
}