﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace BioAPI.Data
{
    /// <summary>
    /// Represents the record in the component registry that defines properties of the BSP installed in the 
    /// system. For security reasons no methods to modify the existing BSPSchema object are provided. 
    /// Instead, an application should use the corresponding method of the DataFactory to create new 
    /// BSPSchema containing modified data.
    /// </summary>
    /// <author>
    /// See CREDITS.txt
    /// </author>
    [Serializable()]
    public class BSPSchema
    {
        public UUID BSPUUID { get; private set; }
        public string BSPDescription { get; private set; }
        public string Path { get; private set; }
        public string SpecVersion { get; private set; }
        public string ProductVersion { get; private set; }
        public string Vendor { get; private set; }
        public List<DataFormat> BSPSupportedFormats { get; private set; }
        public List<BiometricType> FactorsMask { get; private set; }
        public List<BSPSchemaOperations> Operations { get; private set; }
        public List<BSPSchemaOptions> Options { get; private set; }
        public int PayloadPolicy { get; private set; }
        public int MaxPayloadSize { get; private set; }
        public int DefaultVerifyTimeout { get; private set; }
        public int DefaultIdentifyTimeout { get; private set; }
        public int DefaultCaptureTimeout { get; private set; }
        public int DefaultEnrolTimeout { get; private set; }
        public int DefaultCalibrateTimeout { get; private set; }
        public int MaxBSPDbSize { get; private set; }
        public int MaxIdentify { get; private set; }
        public int MaxNumEnrolInstances { get; private set; }
        public byte[] HostingEndpointIRI { get; private set; }
        public UUID BSPAccessUUID { get; private set; }
        public List<DataFormat> BSPSupportedAlgorithms { get; private set; }
        public List<UUID> BSPSupportedTransformOperations { get; private set; }

        public BSPSchema
            (UUID bspUUID, string bspDescription, string path, string specVersion, string productVersion, string vendor,
             List<DataFormat> bspSupportedFormats, List<BiometricType> factorsMask, List<BSPSchemaOperations> operations,
             List<BSPSchemaOptions> options, int payloadPolicy, int maxPayloadSize, int defaultVerifyTimeout, int defaultIdentifyTimeout,
             int defaultCaptureTimeout, int defaultEnrolTimeout, int defaultCalibrateTimeout, int maxBSPDbSize, int maxIdentify,
             int maxNumEnrolInstances, byte[] hostingEndpointIRI, UUID bspAccessUUID, List<DataFormat> bspSupportedAlgorithms,
             List<UUID> bspSupportedTransformOperations)
        {
            BSPUUID = bspUUID;
            BSPDescription = bspDescription;
            Path = path;
            SpecVersion = specVersion;
            ProductVersion = productVersion;
            Vendor = vendor;
            BSPSupportedFormats = bspSupportedFormats;
            FactorsMask = factorsMask;
            Operations = operations;
            Options = options;
            PayloadPolicy = payloadPolicy;
            MaxPayloadSize = maxPayloadSize;
            DefaultVerifyTimeout = defaultVerifyTimeout;
            DefaultIdentifyTimeout = defaultIdentifyTimeout;
            DefaultCaptureTimeout = defaultCaptureTimeout;
            DefaultEnrolTimeout = defaultEnrolTimeout;
            DefaultCalibrateTimeout = defaultCalibrateTimeout;
            MaxBSPDbSize = maxBSPDbSize;
            MaxIdentify = maxIdentify;
            MaxNumEnrolInstances = maxNumEnrolInstances;
            HostingEndpointIRI = hostingEndpointIRI;
            BSPAccessUUID = bspAccessUUID;
            BSPSupportedAlgorithms = bspSupportedAlgorithms;
            BSPSupportedTransformOperations = bspSupportedTransformOperations;
        }
    }
}