﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace BioAPI.Data
{

    /// <summary>
    /// Provides the application the implementation-neutral tool to instantiate BioAPI data types. Because some data types (like UnitProperties
    /// or Candidates) are instantiated purely by the Framework or BSP, the Datafactory does not define methods to instantiate values of such types.
    /// </summary>
    /// <author>
    /// See CREDITS.txt
    /// </author>
    public interface IDataFactory
    {
        IBFPSchema NewBFPProperties(UUID BFPUuid, UnitCategory BFPCategory, string BFPDescription, string path, string specVersion,
            string productVersion, string vendor, IDataFormat[] BFPSupportedFormats, IBiometricType factorsMask, UUID BFPPropertyID, sbyte BFPProperty);

        IBIR NewBIR(IDataFormat _patronFormat, byte _patronHeaderVersion, byte _cBEFFVersion, IDataFormat _bDBFormat, bool _bDBEncription,
            bool _isBound, IBiometricType _factorsMask, IBiometricSubtype _subtype, byte[] _challengeResponse, IDate _bDBCreationDTG, 
            byte[] _bDBIndex, ProcessedLevel _processedLevel, byte[] _bIRIndex, IDataFormat _productID, Purpose _purpose, IPayload _bIRPayload, 
            byte _quality, List<IDate> _bDBValidityPeriod, IDate _creationDTG, byte[] _bIRCreator, List<IDate> _bIRValidityPeriod, 
            IDataFormat _sBFormat, byte[] _biometricData, byte[] _securityBlock);

        IBSPSchema NewBSPProperties(string _BSPDescription, List<IDataFormat> _BSPSupportedFormats, UUID _BSPGuid, int _DefaultCalibrateTimeout,
            int _DefaultCaptureTimeout, int _DefaultEnrolTimeout, int _DefaultIdentifyTimeout, int _DefaultVerifyTimeout,
            List<BiometricTypeInstance> _FactorsMask, long _MaxBSPDbSize, long _MaxIdentify, long _MaxPayloadSize,
            List<BSPSchemaOperationsOperation> _Operations, List<BSPSchemaOptionsOption> _Options, string _Path, int _PayloadPolicy,
            string _ProductVersion, string _SpecVersion, string _Vendor);

        IDate NewDate(int year, int month, int mday, int hour, int minute, int second);
        int NewFMR(int falseMatchRate);
        IIdentifyPopulation NewIdentifyPopulation(System.Collections.Generic.ICollection<IBIR> members);
        IPayload NewPayload(sbyte data);
        IPayload NewPayload(string data);
        IPayload NewPayload(UUID data);
        IBiometricType NewBiometricType(params Type[] types);
        IDataFormat NewBDBFormat(int owner, int type);
        IDataFormat NewSBFormat(int owner, int type);
        IDataFormat NewBIRProductID(int owner, int type);
        IBiometricSubtype NewBiometricSubtype(params BiometricSubtypeInstance[] subject);
        IBSPSchemaOperations NewBSPPropertiesOperations(params BSPSchemaOperationsOperation[] operations);
        IBSPSchemaOptions NewBSPPropertiesOptions(params BSPSchemaOptionsOption[] options);
	    }
}
