﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace BioAPI.Data
{

    /// <summary>
    /// Defines properties of the biometric unit. For security reasons no methods to modify the existing UnitSchema object are provided.
    /// </summary>
    /// <author>
    /// See CREDITS.txt
    /// </author>
    public class UnitSchema
    {
        public UUID BSPUUID { get; set; }
        public UUID UnitManagerUUID { get; set; }
        public int UnitID { get; set; }
        public UnitCategoryType UnitCategory { get; private set; }
        public UUID UnitProperties { get; private set; }
        public string VendorInformation { get; private set; }
        public List<EventKind> SupportedEvents { get; private set; }
        public UUID UnitPropertyID { get; private set; }
        public byte[] UnitProperty { get; private set; }
        public string HardwareVersion { get; private set; }
        public string FirmwareVersion { get; private set; }
        public string SoftwareVersion { get; private set; }
        public string HardwareSerialNumber { get; private set; }
        public bool AuthenticatedHardware { get; private set; }
        public int MaxBSPDbSize { get; private set; }
        public int MaxIdentify { get; private set; }
        public List<SecurityProfileType> SecurityProfile { get; private set; }

        public UnitSchema
            (UUID bspUUID, UUID unitManagerUUID, int unitID, UnitCategoryType unitCategory, UUID unitProperties, string vendorInformation,
             List<EventKind> supportedEvents, UUID unitPropertyID, byte[] unitProperty, string hardwareVersion, string firmwareVersion,
             string softwareVersion, string hardwareSerialNumber, bool authenticatedHardware, int maxBSPDbSize, int maxIdentify,
             List<SecurityProfileType> securityProfile)
        {
            BSPUUID = bspUUID;
            UnitManagerUUID = unitManagerUUID;
            UnitID = unitID;
            UnitCategory = unitCategory;
            UnitProperties = unitProperties;
            VendorInformation = vendorInformation;
            SupportedEvents = supportedEvents;
            UnitPropertyID = unitPropertyID;
            UnitProperty = unitProperty;
            HardwareVersion = hardwareVersion;
            FirmwareVersion = firmwareVersion;
            SoftwareVersion = softwareVersion;
            HardwareSerialNumber = hardwareSerialNumber;
            AuthenticatedHardware = authenticatedHardware;
            MaxBSPDbSize = maxBSPDbSize;
            MaxIdentify = maxIdentify;
            SecurityProfile = securityProfile;
        }
    }
}
