﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BioAPI;
using BioAPI.Data;

namespace BioAPI
{
    /// <summary>
    /// Represents the Biometric Service Provider. The BSP is the factory of Session objects that
    /// provide access to biometric operations. 
    /// </summary>
    /// <author>
    /// See CREDITS.txt
    /// </author>
    public interface IBSP: IArchive, IComparison, IProcessing, ISensor
    {
        BSPSchema Schema { get; }
        BIR BiometricReference { get; }
        byte[] ACBioInstance { get; }

        void BSPAttach(string bioAPIVersion, ACBioParameters acBioParameters, List<UnitListElement> unitList,
            List<SecurityProfileType> securityProfileList);
        void BSPDetach();
        void BSPLoad(BFPEventCallback bioAPINotifyCallback, BFPEnumerationCallback bfpEnumeration);
        void BSPUnload();
        byte CheckQuality(BIR inputBIR, DataFormat qualityAlgorithmID);
        byte[] ControlUnit(int unitID, int controlCode, byte[] inputData);  

        UUID Enrol(BIR capturedBIR, BIR referenceTemplate, List<Purpose> purpose, BiometricSubtype subtype, DataFormat outputFormat,
            byte[] payload, int timeout, List<ResultOptions> options);
        UUID Enrol(List<BIR> capturedBIRs, BIR referenceTemplate, List<Purpose> purpose, BiometricSubtype subtype, DataFormat outputFormat,
            byte[] payload, int timeout, List<ResultOptions> options);
        UUID Enrol(int NumberOfSamples, BIR referenceTemplate, List<Purpose> purpose, BiometricSubtype subtype, DataFormat outputFormat,
            byte[] payload, int timeout, List<ResultOptions> options);
        UUID Enrol(BIR capturedBIR, UUID referenceID, List<Purpose> purpose, BiometricSubtype subtype, DataFormat outputFormat,
            byte[] payload, int timeout, List<ResultOptions> options);
        UUID Enrol(List<BIR> capturedBIRs, UUID referenceID, List<Purpose> purpose, BiometricSubtype subtype, DataFormat outputFormat,
            byte[] payload, int timeout, List<ResultOptions> options);
        UUID Enrol(int NumberOfSamples, UUID referenceID, List<Purpose> purpose, BiometricSubtype subtype, DataFormat outputFormat,
            byte[] payload, int timeout, List<ResultOptions> options);

        List<Candidate> IdentifyAggregated(int maxFMRrequested, BiometricSubtype subtype, bool binning, int maxResults, int timeout,
            List<ResultOptions> options);
        List<BFPListElement> QueryBFPs();
        List<BFPListElement> QueryBFPs(List<UnitCategoryType> unitCategories);
        List<UnitSchema> QueryUnits();
        List<UnitSchema> QueryUnits(List<UnitCategoryType> unitCategories);

        bool VerifyAggregated(int maxFMRrequested, BIR referenceTemplate, BiometricSubtype subtype, int timeout, List<ResultOptions> options);
        bool VerifyAggregated(int maxFMRrequested, BIR processedBIR, UUID referenceKey, BiometricSubtype subtype, int timeout,
            List<ResultOptions> options);
        bool VerifyAggregated(int maxFMRrequested, UUID referenceKey, BiometricSubtype subtype, int timeout, List<ResultOptions> options);

        void SubscribeToGUIEvents(GUISelectEventCallback guiSelectEventCallback, GUIStateEventCallback
            guiStateEventCallback, GUIProgressEventCallback guiProgressEventCallback);
        void UnsubscribeFromGUIEvents();
    }

}
