﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BioAPI;
using BioAPI.Data;

namespace BioAPI
{

    /// <summary>
    /// Represents the biometric system. The biometric system is the hierarchical assembly which root
    /// node is the Framework component. The Framework controls biometric service providers (BSPs)
    /// that drive concrete biometric sensor units and implement various biometric technologies.
    /// To provide necessary services to the Framework BSPs use biometric function provider modules
    /// that are libraries implementing biometric algorithms and/or code that manages sensors hardware.
    /// Along with BSPs, another subordinate of the Framework is component registry that stores the
    /// information about BSPs and BFPs that areavailable in the biometric system.
    /// </summary>
    /// <author>
    /// See CREDITS.txt
    /// </author>
    public interface IFramework : IComponentRegistry
    {

        /// <summary>
        /// Retrieves the information about the biometric system. 
        /// the FrameworkSchema object that identifies the implementation of the BioAPI framework.        
        /// </summary>
        FrameworkSchema FrameworkSchema { get; }
        List<AttachedSession> AttachedSessionList { get; }

        AttachedSession BSPAttach (UUID bspUUID, string bioAPIVersion, ACBioParameters acBioParameters, List<UnitListElement> unitList, 
            List<SecurityProfileType> securityProfileList);
        void BSPDetach (AttachedSession sessionAttached);
        void BSPLoad (UUID bspID, BFPEventCallback notifyCallback, BFPEnumerationCallback bfpEnumeration, string context);

        void BSPUnload(UUID bspID, BFPEventCallback notifyCallback, BFPEnumerationCallback bfpEnumeration, string context);

        void EnableEventNotifications (UUID bspID, List<EventKind> events);
        List<BFPSchema> EnumBFPs ();
        List<BSPSchema> EnumBSPs ();
        void Init (string version);
        List<BFPListElement> QueryBFPs (UUID bspUUID);
        List<UnitSchema> QueryUnits (UUID bspUUID);
        void Terminate();
    }
}
