﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;/**/

namespace BioAPI
{
    /************************************************************************************************************ 
     * Represents the callback that a biometric application supplies to allow a BSP to stream data for display, *
     * in the form of a sequence of bitmaps, to the application through the Framework.                          *
     *                                                                                                          *
     * @author See CREDITS.txt                                                                                  *
     ***********************************************************************************************************/

    public interface GUIImageObserve
    {
        /**
	     * A callback function that a biometric application supplies to allow a BSP to stream data for display,
	     * in the form of a sequence of bitmaps, to the application through the Framework.
	     * 
	     * @param	image the bitmap to be displayed.
	     */
        void imageUpdate(GUIImageObserve_Bitmap image);
    }

    /**
	 * Represents the graphic for display by an application.
	 * 
	 */
    public interface GUIImageObserve_Bitmap
    {
        /**
		 * @return	the height of the graphic in pixels.
		 */
        int getHeight();

        /**
		 *@return	a series of 8-bit grayscale pixels (where 00=black and FF=white), read from left to right, top to bottom).
		 */
        byte[] getPixels();

        /**
		 * @return	a series of 8-bit grayscale pixels (where 00=black and FF=white), read from left to right, top to bottom).
		 */
        Stream getPixelsAsStream();

        /**
		 * @return	the width of the graphic in pixels.
		 */
        int getWidth();
    }

}
