﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BioAPI.Data;

namespace BioAPI
{

    /// <summary>
    /// Represents the group of biometric operations that are available when the unit of SENSOR category is
    /// linked to the attach session.
    /// </summary>
    /// <author>
    /// See CREDITS.txt
    /// </author>
    public interface ISensor
    {
        UnitSchema UnitSchema { get; }
        byte[] ACBioInstance { get; }
        byte[] AuxiliaryData { get; }
        UnitIndicatorStatus IndicatorStatus { get; }

        /// <summary>
        /// Captures samples for the purpose specified, and the BSP returns either an "intermediate" type BIR (if the
        /// Processing.Process() method needs to be called), or a "processed" BIR (if not). The purpose is recorded in
        /// the header of the captured BIR. If RequestAuditData option is specified a BIR of type "raw" may be returned
        /// in CaptureResult. By default, the BSP is responsible for providing the user interface associated with the
        /// capture operation. The application may, however, request control of the GUI "look-and-feel" by providing a
        /// GUI callback vis the Session.SetGUIObservers() method. Capture serializes use of the sensor device. If two
        /// or more biometric applications are racing for the sensor, the losers will wait until the operation completes
        /// or the timeout expires. This serialization takes place in all functions that capture data. The BSP is
        /// responsible for serializing. It may do this by either throwning an exception to indicate that device is
        /// busy or by queuing requests.
        /// </summary>
        /// <param name="purpose">
        /// indicates the purpose of the biometric data capture. 
        /// </param>
        /// <param name="subtype">
        /// specifies which subtype (e.g., left/right eye) to capture. null indicates that the value is not provided.
        /// </param>
        /// <param name="outputFormat">
        /// specifies which BDB format to use for the returned processed BIR, if the BSP supports more than one format.
        /// null indicates that the BSP is to select the format. 
        /// </param>
        /// <param name="timeout">
        /// an integer specifying the timeout value (in milliseconds) for the operation. If this timeout is reached an
        /// exception is thrown. This value can be any positive number. A -1 value means the BSPs default timeout value
        /// will be used. 
        /// </param>
        /// <param name="options">
        /// requests additional output such as audit data. 
        /// </param>
        /// <returns>
        /// the CaptureResult object that represents the result of capture operation.
        /// </returns>

        BIR Capture(List<Purpose> purpose, BiometricSubtype subtype, DataFormat outputFormat, int timeout, List<ResultOptions> options);

        /// <summary>
        /// Performs a calibration of the attached
        /// sensor BioAPI Unit if that sensor unit supports it.
        /// </summary>
        /// <param name="timeout">
        /// an integer specifying the timeout value (in milliseconds) for the operation. If this timeout is reached an
        /// exception is thrown. This value can be any positive number. A -1 value means the BSPs default timeout value
        /// will be used. 
        /// </param>
        void Calibrate(int timeout);

        void SetIndicatorStatus (UnitIndicatorStatus indicatorStatus);

        void SetPowerMode (UnitPowerMode powerMode);
    }
}
