﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Runtime.Serialization;
using System.Reflection;
using System.Xml;
using BioAPI;
using BioAPI.Data;

namespace Framework
{
    public class FrameworkImplemented : IFramework
    {
        private string path = @"..\..\..\..\Framework\ComponentRegistry";
        private bool isFrameworkInit = false;
        private DataContractSerializer serbsp = new DataContractSerializer(typeof(BSPSchema));
        private DataContractSerializer serbfp = new DataContractSerializer(typeof(BFPSchema));
        private List<BSPSchema> bsplist = new List<BSPSchema>();
        private List<BFPSchema> bfplist = new List<BFPSchema>();
        private IBSP bsptest = null;
        private FrameworkSchema frameworkSchema = null;
        private List<AttachedSession> attachedSessionList = null;
        private AttachedSession attachedsession = null;

        public FrameworkSchema FrameworkSchema 
        {
            get 
            {
                return frameworkSchema;
            }
        }

        public List<AttachedSession> AttachedSessionList 
        { 
            get 
            { 
                return attachedSessionList; 
            } 
        }

        public AttachedSession BSPAttach(UUID bspUUID, string bioAPIVersion, ACBioParameters acBioParameters, List<UnitListElement> unitList,
            List<SecurityProfileType> securityProfileList)
        {
            for (int i = 0; i < bsplist.Count; i++)
            {
                if (bspUUID == bsplist[i].BSPUUID)
                {
                    attachedsession = new AttachedSession(bsptest);
                }
            }

            bsptest.BSPAttach(bioAPIVersion, acBioParameters, unitList, securityProfileList);
            return attachedsession;
        }

        public void BSPDetach(AttachedSession sessionAttached)
        {

        }

        public void BSPLoad(UUID bspID, BFPEventCallback notifyCallback, BFPEnumerationCallback bfpEnumeration, string context)
        {
            for (int i = 0; i < bsplist.Count; i++)
            {
                if (bspID == bsplist[i].BSPUUID)
                {
                    Assembly a = Assembly.LoadFrom(bsplist[i].Path);
                    foreach (Type type in a.GetTypes())
                    {
                        if (type.IsClass && typeof(IBSP).IsAssignableFrom(type))
                        {
                            bsptest = (IBSP)Activator.CreateInstance(type);
                        }
                    }
                }
            }

            bsptest.BSPLoad(notifyCallback, bfpEnumeration);
        }

        public void BSPUnload(UUID bspID, BFPEventCallback notifyCallback, BFPEnumerationCallback bfpEnumeration, string context)
        {

        }
        public void EnableEventNotifications(UUID bspID, List<EventKind> events)
        {

        }

        public List<BFPSchema> EnumBFPs() 
        {
            if (isFrameworkInit == false)
                throw new BioAPIException(BioAPIException.Facility.Framework, BioAPIException.BioAPIErrFrameworkNotInitialized,
                    Properties.Resources.stringFwNotInicializated);
            
            return bfplist;
        }

        public List<BSPSchema> EnumBSPs() 
        {
            if (isFrameworkInit == false)
                throw new BioAPIException(BioAPIException.Facility.Framework, BioAPIException.BioAPIErrFrameworkNotInitialized,
                     Properties.Resources.stringFwNotInicializated);

            return bsplist; 
        }

        public void Init(string version)
        {
            isFrameworkInit = true;
            int bspCount = Directory.GetFiles(Path.Combine(path, "BSPs"), "*.xml*", SearchOption.TopDirectoryOnly).Length;
            int bfpCount = Directory.GetFiles(Path.Combine(path, "BFPs"), "*.xml*", SearchOption.TopDirectoryOnly).Length;
            string[] bspPaths = Directory.GetFiles(Path.Combine(path, "BSPs"), "*.xml", SearchOption.TopDirectoryOnly);
            string[] bfpPaths = Directory.GetFiles(Path.Combine(path, "BFPs"), "*.xml", SearchOption.TopDirectoryOnly);

            for (int i = 0; i < bspCount; i++)
            {
                using (FileStream fs = new FileStream(bspPaths[i], FileMode.Open))
                {
                    BSPSchema bspsch = (BSPSchema)serbsp.ReadObject(fs);
                    bsplist.Add(bspsch);
                }
            }

            for (int i = 0; i < bfpCount; i++)
            {
                using (FileStream fs = new FileStream(bfpPaths[i], FileMode.Open))
                {
                    BFPSchema bfpsch = (BFPSchema)serbfp.ReadObject(fs);
                    bfplist.Add(bfpsch);
                }
            }   
        }

        public List<BFPListElement> QueryBFPs(UUID bspUUID) 
        {
            for (int i = 0; i < bsplist.Count; i++)
            {
                if (bspUUID == bsplist[i].BSPUUID)
                {
                    return bsptest.QueryBFPs();
                }
                else
                {
                    throw new BioAPIException(BioAPIException.Facility.Framework, BioAPIException.BioAPIErrBspNotLoaded,
                        Properties.Resources.stringBSPNotLoaded);
                }
            }
            return null;
        }

        public List<UnitSchema> QueryUnits(UUID bspUUID) 
        {
            for (int i = 0; i < bsplist.Count; i++)
            {
                if (bspUUID == bsplist[i].BSPUUID)
                {
                    return bsptest.QueryUnits();
                }
                else
                {
                    throw new BioAPIException(BioAPIException.Facility.Framework, BioAPIException.BioAPIErrBspNotLoaded,
                        Properties.Resources.stringBSPNotLoaded);
                }
            }
            return null;
        }

        public void Terminate()
        {

        }

        //////////////////////////////////////////////////////////////////

        public void InstallBFP(BFPSchema bfpSchema, bool update)
        {
            try
            {
                string finalpath = Path.Combine(path, "BFPs", bfpSchema.Vendor + "." + bfpSchema.ProductVersion + ".xml");
                using (FileStream file = new FileStream(finalpath, FileMode.Create))
                {
                    serbfp.WriteObject(file, bfpSchema);
                }
            }
            catch (BioAPIException)
            {
                throw new BioAPIException (BioAPIException.Facility.BSP, BioAPIException.BioAPIBspError,
                    Properties.Resources.stringErrDuringInstalation);
            }
        }

        public void InstallBSP(BSPSchema bspSchema, bool update)
        {
            try
            {
                string finalpath = Path.Combine(path, "BSPs", bspSchema.Vendor + "." + bspSchema.ProductVersion + ".xml");
                using (FileStream file = new FileStream(finalpath, FileMode.Create))
                {
                    serbsp.WriteObject(file, bspSchema);
                }
            }
            catch (BioAPIException)
            {
                throw new BioAPIException(BioAPIException.Facility.BSP, BioAPIException.BioAPIBspError,
                     Properties.Resources.stringErrDuringInstalation);
            }
        }

        public void UninstallBFP(UUID bfpUUID)
        {

        }

        public void UninstallBSP(UUID bspUUID)
        {

        }
    }
}
